#!/usr/local/bin/python2.7
'''Basics of strings, lists, and printing.

For a complete list of string methods, see:
    http://docs.python.org/library/stdtypes.html#index-634

For a complete list of operations you can do on lists (and other sequence
types), see:
    http://docs.python.org/library/stdtypes.html#typesseq-mutable

Made by Andrew Uzilov for BME205, Fall 2011.
Email questions, concerns, etc. to:
auzilov@ucsc.edu
'''

### STRINGS

# Create a new string.
myString = "I'm a simple string."
print myString

# Another string.
keyword = 'simple'

# How long is the keyword?
l = len (keyword)
print 'The keyword has', l, 'characters.'

# Getting specific characters from a string.
print 'First:',  keyword[0]
print 'Last:',   keyword[-1]
print 'Middle:', keyword[2]

# A neat way to print several variables.
print 'The word "%s" has %d characters.' % (keyword, l)

# Find where the keyword begins in 'myString'.
i = myString.find (keyword)
print 'The keyword "%s" begins at index %d.' % (keyword, i)

# Get substrings using slices.
print 'Slice:', myString[0:3]
print 'Slice:', myString[i:i+l]
print 'Slice:', myString[:i]  # everything before 'i'
print 'Slice:', myString[i:]  # everything after 'i'

# More tricks with slices.
print myString[i:i+l:2]  # every other character
print myString[::2]
print myString[::-1]  # reverse a string

# Slices DO NOT raise out-of-bounds exceptions!
# (But indexing does.)
print 'Far out:', myString[i:1000]

# Printing the same string multiple times.
print 'xyz' * 3


### LISTS

# An empty list.
myList = []

# Adding to a list.
myList.append ('item0')

# Lists can mix types.
myList = [None, 'item0', 1, 20.5, [5, 6, 7]]
print myList

# Lists can be indexed and sliced just like strings;
# both are sequence types...
print myList[1]
print myList[:-2]
print myList[-1][2]

# ...except lists are MUTABLE sequence types.
myList[2:4] = [keyword, keyword, keyword]
print myList


### PUTTING IT TOGETHER

# Splitting strings to make lists.

bedString = 'chrX	152363371	152364082	uc010nuc.1	237	-'
fields = bedString.split()  # split on whitespace
print fields
print 'Split into', len (fields), 'fields.'
print 'Do some math on input:', int (fields[2]) - int (fields[1])


# Strings are immutable.
# So how do we "change" strings?
# Convert to a list first.

myStringAsList = list (myString)
print myStringAsList

myStringAsList[i:i+l] = list ('complex')
print myStringAsList

myFinalString = '_'.join (myStringAsList)
print myFinalString

# But what you really want is this.
myFinalString = ''.join (myStringAsList)
print myFinalString

