#!/usr/local/bin/python2.7
'''Command-line argument parsing and control flow.

Made by Andrew Uzilov for BME205, Fall 2011.
Email questions, concerns, etc. to:
auzilov@ucsc.edu
'''

import argparse
import random
import sys

# Parse command-line arguments.
parser = argparse.ArgumentParser (description='A die-rolling program.')
parser.add_argument ('-n', '--numsides', action='store', dest='numSides',
                     type=int, required=True, metavar='INT',
                     help='number of sides that the die has')
args = parser.parse_args()

# Do a safety check.
if args.numSides < 4:
    print >> sys.stderr, "Can't have a die with less than 4 sides.  Quitting..."
    sys.exit (1)

# Roll the die!
result1 = random.randint (1, args.numSides)
result2 = random.randint (1, args.numSides)
print 'You rolled %d and %d on a %d-sided die.' \
    % (result1, result2, args.numSides)


### CONDITIONALS

if result1 == result2:
    print 'Both rolls are the same!'

if result1 % 2 == 0:
    print 'First roll was an even number.'
else:
    print 'First roll was an odd number.'

if result1 % 2 == 0 and result2 % 2 == 0:
    print 'First roll even, second roll even.'
elif result1 % 2 == 0 and result2 % 2 != 0:
    print 'First roll even, second roll odd.'
elif result1 % 2 != 0 and result2 % 2 == 0:
    print 'First roll odd, second roll even.'
else:
    print 'First roll even, second roll even.'

if result1 % 2 == result2 % 2:
    if result1 % 2 == 0:
        print 'Both rolls were odd.'
    else:
        print 'Both rolls were even.'


### LOOPS

# Long way to roll the die 10 times and display some results.

results = []
for i in xrange (10):
    results.append (random.randint (1, args.numSides))

print 'Roll results:'
total = 0
for result in results:
    print result
    total += result  # shorthand for total = total + result
print 'Total of rolls is:', total

# Much shorter, more 'Pythonic' way to do all of the above.
# Use list comprehensions, string methods, and built-in functions.

results = [random.randint (1, args.numSides) for i in xrange (10)]
print 'Roll results:', \
    ', '.join (map (str, results))
print 'Total of rolls is:', sum (results)


