#!/usr/local/bin/python2.7
'''Putting it all together and adding functions: a simple BED parser.

Made by Andrew Uzilov for BME205, Fall 2011.
Email questions, concerns, etc. to:
auzilov@ucsc.edu
'''

import argparse
import sys


def getGeneName (bedLine):
    '''Extracts the gene name from a BED line and returns it, doing some sanity
    checks.
    '''
    fields = bedLine.split ('\t')  # split line on tab character
    assert len (fields) >= 4, \
        "Bad BED line!  Not enough fields.  Offending line:\n%s" % bedLine
    return fields[3]


def loadGeneNames (bedFilePath):
    '''Loads gene names from a BED file and returns them as a list.
    '''
    ret = []
    for line in open (bedFilePath, 'r'):
        if line.startswith ('track'):
            # Skip header lines.
            continue
        else:
            line = line.rstrip ('\n\r')  # usually a good idea
            ret.append (getGeneName (line))
    return ret

    
if __name__ == '__main__':
    parser = argparse.ArgumentParser (
        description='Print the gene names from a BED file.')
    parser.add_argument ('-b', '--bed', action='store', dest='bedFilePath',
                         required=True, metavar='FILENAME',
                         help='input BED file')
    args = parser.parse_args()

    geneNames = loadGeneNames (args.bedFilePath)
    print >> sys.stderr, 'Loaded', len (geneNames), 'gene names.'
    for gene in geneNames:
        print gene
