#!/usr/local/bin/python2.7
'''Some "gotchas" (sources of bugs/frustration for newcomers to Python)
and other miscellaneous stuff.

This WILL NOT run as a complete program like the other examples, it is just
informative code snippets.

Made by Andrew Uzilov for BME205, Fall 2011.
Email questions, concerns, etc. to:
auzilov@ucsc.edu
'''

#------------------------------------------------------------------------------

# DO NOT use Python built-in function names to name your data structures!
# Python will allow you to do this, but it will cause strange and mysterious
# errors down the line.
# So, DO NOT do this:

list = [1, 2, 3]  # oops!  you just rebound a built-in function
anotherList = list ('some chars')  # now this won't work

# Python built-in function names can be found here:
#     http://docs.python.org/library/functions.html


#------------------------------------------------------------------------------

# When comparing if an object is None, do this:
if someObj is None:
    doSomething()

# DO NOT do this:
if someObj == None:
    doSomething()

# For an explanation, read:
#     http://jaredgrubb.blogspot.com/2009/04/python-is-none-vs-none.html


#------------------------------------------------------------------------------

# There is a recently popularized construct (in Python 2.6 and later only)
# for reading from/writing to files.
# It uses a concept called "statement context managers".
# It looks like this:

with open ('out.txt', 'w') as outFile:
    print >> outFile, 'Starting...'
    print >> outFile, someOtherStuff()
    print >> outFile, 'Done!'

# The above is basically a safe way to make sure your file is always closed even
# if something goes wrong in the function body.
# For FILE OBJECTS ONLY, it is sort of equivalent to this:

outFile = open ('out.txt', 'w')
try:
    print >> outFile, 'Starting...'
    print >> outFile, someOtherStuff()
    print >> outFile, 'Done!'
finally:
    outFile.close()

# Except that context managers ("with" keyword) have extra power not really
# highlighted by this example.


#------------------------------------------------------------------------------

# If you bind/rebind a name in a function, it becomes local to the function.
# Guess what this prints?

myString = 'old school'
def tryToChangeString (newString):
    myString = newString
    print 'tried to change to:', myString

print 'Used to be:', myString
tryToChangeString ('new school')
print 'Now it is:', myString


#------------------------------------------------------------------------------

# This is the WRONG way to pre-initialize a list of empty lists:

lists = [[]] * 3

# Guess what happens now?

lists[0].append (3)
print lists

# Here is the right way to do it:

lists = [[] for i in xrange(3)]
lists[0].append (3)
lists[1].append (5)
lists[2].append (7)
print lists


#------------------------------------------------------------------------------

# When creating functions with named default parameters, DO NOT do this:

def myFunc (inParam=[]):
    print 'Got:', inParam
    inParam.append (311)

myFunc()
myFunc()
myFunc()

# This is a mistake commonly made in constructor (__init__) functions to
# objects.
# Don't do this.
# Instead, do this:

def myFunc (inParam=None):
    if inParam is None:
        inParam = []  # now, each function call has ITS OWN list to play with
    print 'Got:', inParam
    inParam.append (311)

myFunc()
myFunc()
myFunc()

